/* ƬФ tool
 * funa@nolm01.iee.niit.ac.jp, shige
 * 02/24 2001
 * auslcut.c
 * 
 * $Id: auslcut.c,v 1.2 2017-03-06 17:07:06+09 shige Exp $
 */
#ifndef __AUSLCUT_C__
#define __AUSLCUT_C__

#include "auslcut.h"

int main(int argc, char *argv[])
{
    int level, cpos[MaxLevel];
    int size, ctop[MaxLevel], clast[MaxLevel];
    int pos[MaxDivide + 1];
    int key, k;
    int skip = 0;
    int ret;
    int divide, verbose, putkey;
    int sndblock, sndvalue;
    double minvalue;
    char *fname, *outfname;
    int mode;
#ifdef USEPLOTSOUND
    int plotsnd;
#endif
    char tmpfnames[MaxDivide + 1][MaxBuff];

    /* command line μ */
    ret = arganal(argc, argv, &divide, &minvalue, &sndvalue, &verbose,
		  &putkey, &mode,
#ifdef USEPLOTSOUND
		  &plotsnd,
#endif
		  &TempDir, &HeadSound, &NullSound, &PlayComm,
		  &BackwardPlay, &SoundFormat, &fname, &outfname);
    if (ret) { usage(argv[0]); return 1; }
#ifdef DEBUG1
    fprintf(stderr, "ret = %d\n", ret);
    fprintf(stderr, "mode       = %d\n", mode);
    fprintf(stderr, "divide     = %d\n", divide);
    fprintf(stderr, "minvalue   = %f\n", minvalue);
    fprintf(stderr, "sndvalue   = %d\n", sndvalue);
    fprintf(stderr, "verbose    = %d\n", verbose);
    fprintf(stderr, "putkey     = %d\n", putkey);
#ifdef USEPLOTSOUND
    fprintf(stderr, "plotsnd    = %d\n", plotsnd);
#endif
    fprintf(stderr, "tmpdir     = %s\n", TempDir);
    fprintf(stderr, "headsnd    = %s\n", HeadSound);
    fprintf(stderr, "nullsnd    = %s\n", NullSound);
    fprintf(stderr, "sndcmd     = %s\n", PlayComm);
    fprintf(stderr, "backward   = %d\n", BackwardPlay);
    fprintf(stderr, "sndform    = ");
    switch (SoundFormat) {
    case E_AU: fprintf(stderr, "AU\n"); break;
    case E_WAV: fprintf(stderr, "WAV\n"); break;
    case E_AIFF: fprintf(stderr, "AIFF\n"); break;
    default: fprintf(stderr, "Unknown.\n");
    } 
    fprintf(stderr, "fname      = %s\n", fname);
    fprintf(stderr, "outfname   = %s\n", outfname);
#endif /* DEBUG1 */


    /* ѿgraphics ν */
    if ((ret = setinit(mode, divide, tmpfnames, fname, &outfname, 
		       &size, &sndblock, minvalue, sndvalue,
#ifdef USEPLOTSOUND
		       plotsnd,
#endif
		       verbose)) < 0) {
	if (verbose) fprintf(stderr, "Initialize error. (%d)\n", ret);
	return 2;
    }
#ifdef DEBUG1
    fprintf(stderr, "outfname   = %s\n", outfname);
    fprintf(stderr, "size       = %d\n", size);
    fprintf(stderr, "sndblock   = %d\n", sndblock);
    fprintf(stderr, "Sample Rate= %f\n", SamplingRate);
    fprintf(stderr, "MinimumSize= %f\n", MinimumSize);
    
#endif /* DEBUG1 */

    if (ret == 0) {
#ifdef USEPLOTSOUND
	if (plotsnd) plotcontrol(mode, -1, NULL, 0, NULL, 0);
#endif
	if (verbose) fprintf(stderr, "Quit.\n");
	return 0;
    }
    level = 0;
    ctop[level] = 0; clast[level] = size;

    while (1) { /* plot & play loop */
	if (level + 1 >= MaxLevel) {
	    if (verbose) fprintf(stderr, "Too much level.\n"); 
	    atlast(outfname, fname, mode, level, divide, verbose,
		   ctop[level], ctop, clast, cpos, tmpfnames);
	    return 0;
	}
	if (setpos(pos, divide, ctop[level], clast[level]) < 0) { 
	    if (verbose) fprintf(stderr, "Too small cell size.\n"); 
	    atlast(outfname, fname, mode, level, divide, verbose,
		   ctop[level], ctop, clast, cpos, tmpfnames);
	    return 0;
	}
#ifdef PLOTSOUND
	if (plotsnd) {
	    if (verbose) fprintf(stderr, "Plotting...");
	    plotcontrol(mode, 1, NULL, divide + 1, pos, 0);
	    if (verbose) fprintf(stderr, "completed.\n");
	}
#endif
	while (1) { /* play sound loop */
	    if (verbose) fprintf(stderr, "Putting sound...");
	    putsound(skip, fname, divide + 1, pos, size, sndblock,
		     tmpfnames);
	    skip = 0;
	    if (verbose) fprintf(stderr, "completed.\n");
	    key = getkey(putkey, divide);
	    if (key == '\n') {
		skip = 1; continue;
	    } else if (key >= '1' && key <= divide + '0') {
		k = key - '0';
		cpos[level] = k;
		level ++;
		ctop[level] = pos[k - 1];
		clast[level] = pos[k];
		break;
	    } else if (key == 'q' || key == EOF) {
		atlast(outfname, fname, mode, level, divide, verbose,
		       pos[divide - 1], ctop, clast, cpos, tmpfnames);
		return 0;
	    } else if (key == '\n') { /* Do nothing */ }
	    else if (key == 'p') {
		if (-- level < 0) {
		    if (verbose)
		      fprintf(stderr, "This is TOP level.\n");
		    level = 0;
		}
		break;
	    } else if (key == 'b') {
		if (level == 0) { /* TOP level */
		    if (verbose) {
			fprintf(stderr, "Cannot go backward");
			fprintf(stderr, 
				" because this is TOP level.\n");
		    }
		    atlast(outfname, fname, mode, level, divide,
			   verbose, ctop[level], ctop, clast, cpos,
			   tmpfnames);
		    return 0;
		} else if (cpos[level - 1] <= 1) { /* ʤ */
		    if (verbose) {
			fprintf(stderr, "This is FIRST position.");
			fprintf(stderr,
				" Up to One PREVIOUS level.\n");
		    }
		    level --;
		    break;
		} else {
		    k = -- cpos[level - 1];
		    setpos(pos, divide, ctop[level - 1], 
			   clast[level - 1]);
		    ctop[level] = pos[k - 1];
		    clast[level] = pos[k];
		    break;
		}
	    }
	    else if (key == 'f') {
		if (level == 0) { /* TOP level *//* ¿ʬʤ */
		    if (verbose) {
			fprintf(stderr, "Cannot go foreward");
			fprintf(stderr, 
				" because this is TOP level.\n");
		    }
		    atlast(outfname, fname, mode, level, divide,
			   verbose, clast[level], ctop, clast, cpos,
			   tmpfnames);
		    return 0;
		} else if (cpos[level - 1] >= divide) {
		    /* ˿ʤʤ */
		    if (verbose) {
			fprintf(stderr, "This is LAST position.");
			fprintf(stderr,
				" Up to One PREVIOUS level.\n");
		    }
		    level --;
		    break;
		} else {
		    k = ++ cpos[level - 1];
		    setpos(pos, divide, ctop[level - 1],
			   clast[level - 1]);
		    ctop[level] = pos[k - 1];
		    clast[level] = pos[k];
		    break;
		}
	    }
	} /* play sound loop */
    }
}


/* ##########  ########## */
/* mode :  1 ==> millisec, 0 ==> block */
int setinit(int mode, int divide, char tmpfnames[][MaxBuff],
	    char *fname, char **Outf, int *size, int *sndblock,
	    double minvalue, int sndvalue,
#ifdef USEPLOTSOUND
	    int plotsnd,
#endif
	    int verbose)
{
    FILE *tmpf;

    mktmpfnames(divide, tmpfnames);
    if ((SamplingRate = getsamplingrate(fname)) < 0) return -1;
    if ((*size = getsoundblocks(fname)) < 0.0) return -1;
    if (minvalue < 0) MinimumSize = (mode) ? ms2blck(MINLEN) : MINSIZE;
    else MinimumSize = (mode) ? ms2blck(minvalue) : minvalue;
    if (sndvalue < 0) *sndblock = (mode) ? int45(ms2blck(SNDLEN)) 
			  : SNDBLOCK;
    else *sndblock = (mode) ? int45(ms2blck(sndvalue)) : sndvalue;
    if (*sndblock <= MinimumSize) return -1;
#ifdef USEPLOTSOUND
    if (plotsnd) {
	if (verbose) fprintf(stderr, "Plotting...");
	if (plotcontrol(mode, 0, fname, 0, NULL, *size) < 0) return -2;
	if (verbose) fprintf(stderr, "completed.\n");
    }
#endif
    if (verbose) fprintf(stderr, "Hit return key.\n");
    if (getchar() == 'q') return 0;
    rewind(stdin);

    if (strcmp(*Outf, "+") == 0) *Outf = NULL;
    else if (strcmp(*Outf, "-") != 0) {
	if ((tmpf = fopen(*Outf, "w")) == NULL) return -3;
	fclose(tmpf);
    }
    return 1;
}

/* ########## temporary file ̾κ ########## */
void mktmpfnames(int divide, char tmpfnames[][MaxBuff])
{
    int j;

    for (j=1; j<=divide+1; j++)
      sprintf(tmpfnames[j - 1], "%s/%s%d.%s", TempDir, TempFnameHead,
	      j, auext[SoundFormat]);
}

/* ##########  ########## */
/* ausubstr ˤڤФauconn ˤ硢auplay ˤ */
void putsound(int skip, char *fname, int N, int pos[], int size,
	      int sndblock, char tmpfnames[][MaxBuff])
{
    char buff[MaxBuff];
    char buff2[MaxBuff];
    int j;
    int start, last;

    sprintf(buff2, "%s -Lb%d+", auconn[SoundFormat], sndblock);
    for (j=1; j<=N; j++) {
	if (BackwardPlay) {
	    if (size - pos[j - 1] >= MinimumSize) {
		if (!skip) {
		    sprintf(buff, "%s %d %s > %s",
			    ausubstr[SoundFormat], pos[j - 1], fname,
			    tmpfnames[j - 1]);
#ifdef DEBUG2
		    fprintf(stderr, "DEBUG2: %s\n", buff);
#endif
		    system(buff);
		}
		sprintf(buff2, "%s %s %s", buff2, HeadSound,
			tmpfnames[j - 1]);
	    } else
	      sprintf(buff2, "%s %s %s", buff2, HeadSound, NullSound);
	} else {
	    if (pos[j - 1] >= MinimumSize) {
		if (! skip) {
		    if (pos[j - 1] > sndblock) { 
			start = pos[j - 1] - sndblock; last = sndblock;
		    } else { start = 0; last = pos[j - 1]; }
		    sprintf(buff, "%s %d %d %s > %s",
			    ausubstr[SoundFormat], start, last, fname,
			    tmpfnames[j - 1]);
#ifdef DEBUG2
		    fprintf(stderr, "DEBUG2: %s\n", buff);
#endif
		    system(buff);
		}
		sprintf(buff2, "%s %s %s", buff2, HeadSound,
			tmpfnames[j - 1]);
	    } else
	      sprintf(buff2, "%s %s %s", buff2, HeadSound, NullSound);
	}
	/* ʤ󤫽ʣƤʬ뤬ޤ㡼ʤ */
    }
    sprintf(buff2, "%s %s %s", buff2, NullSound, PlayComm);
#ifdef DEBUG2
    fprintf(stderr, "DEBUG2: %s\n", buff2);
#endif
    system(buff2);
}

/* ########## ʬ ########## */
int setpos(int pos[], int divide, int ctop, int clast)
{
    double dx;
    int j;

    if ((dx = (double)(clast - ctop)/divide) < MinimumSize) {
#ifdef DEBUG3
	fprintf(stderr, "DEBUG3: (ctop, clast, MinimumSize, dx)");
	fprintf(stderr, " = (%d, %d, %f, %f)\n", ctop, clast, 
		MinimumSize, dx);
#endif
	return -1;
    }
    pos[0] = ctop;
    for (j=1; j<divide; j++) pos[j] = ctop + int45(j*dx);
    pos[divide] = clast;
    return divide;
}

/* ########## ǽ ########## */
void atlast(char *outf, char *fname, int mode, int level, int divide,
	    int verbose, int pos, int ctop[], int clast[], int cpos[],
	    char tmpfnames[][MaxBuff])
{
    int j;

    if (verbose)
      puthistory(mode, divide, level, pos, ctop, clast, cpos);
    outputresult(outf, fname, pos);
    for (j=1; j<=divide+1; j++) remove(tmpfnames[j - 1]);
#ifdef USEPLOTSOUND
    plotcontrol(mode, -1, NULL, 0, NULL, 0);
#endif
}


/* at last β
 * mode: 1 ==> millisec, 0 ==> block
 */ 
void puthistory(int mode, int divide, int level, int pos, int 
		ctop[], int clast[], int cpos[])
{
    int j;
    double dx;
    int x;

    fprintf(stderr, "HISTORY:\n");
    for (j=0; j<level; j++) {
	if (mode)
	  fprintf(stderr, " [%d]: (TOP, LAST) = (%f, %f), ", j, 
		  blck2ms(ctop[j]), blck2ms(clast[j]));
	else
	  fprintf(stderr, " [%d]: (TOP, LAST) = (%d, %d), ", j, 
		  ctop[j], clast[j]);
	dx = (double)(clast[j] - ctop[j])/divide;
	x = ctop[j] + int45(cpos[j]*dx);
	if (mode)
	  fprintf(stderr, " CHOOSE = %d (%f)\n", cpos[j], blck2ms(x));
	else fprintf(stderr, " CHOOSE = %d (%d)\n", cpos[j], x);
    }
    if (mode) {
	fprintf(stderr, " [%d]: (TOP, LAST) = (%f, %f)\n",
		level, blck2ms(ctop[level]), blck2ms(clast[level]));
	fprintf(stderr, " RESULT: %f\n", blck2ms(pos));
    } else {
	fprintf(stderr, " [%d]: (TOP, LAST) = (%d, %d)\n",
		level, ctop[level], clast[level]);
	fprintf(stderr, " RESULT: %d\n", pos);
    }
}

/* at last β */
void outputresult(char *outfname, char *fname, int pos)
{
    char buff[MaxBuff];

    if (outfname == NULL) return;
    if (strcmp(outfname, "-") == 0)
      sprintf(buff, "%s %d %s", ausubstr[SoundFormat], pos, fname);
    else
      sprintf(buff, "%s %d %s > %s", ausubstr[SoundFormat], pos, fname,
	      outfname);
    system(buff);
}

/* ########## key  ########## */
int getkey(int putkey, int divide)
{
    int get;
    
    while (1) { /* key input loop */
	if (putkey) {
	    fprintf(stderr, "Hit KEY + return)\n");
	    fprintf(stderr, " KEY: 1, 2, ... ==> choose 1, 2, ...\n");
	    fprintf(stderr, "      q       ==> quit\n");
	    fprintf(stderr, "      nothing ==> again\n");
	    fprintf(stderr, "      p       ==> previous level\n");
	    fprintf(stderr, "      b       ==> backward\n");
	    fprintf(stderr, "      f       ==> forward\n");
	}
	get = getchar();
	rewind(stdin);
	if (get == 'q' || get == '\n' || get == 'p' || get == 'b' 
	    || get == 'f') return get;
	if (get >= '1' && get <= divide + '0') return get;
	if (get == EOF) return get;
    }
}

/* ########## command line  ########## */
/* ʤѤ˽񤱤夳Ȥ*/
int arganal(int argc, char *argv[], int *Divide, double *MinValue,
	    int *SndValue, int *Verbose, int *Putkey, int *UseMilliSec,
#ifdef USEPLOTSOUND
	    int *PlotSnd,
#endif
	    char **TmpDir, char **HeadSnd, char **NullSnd,
	    char **SndCmd, int *Backward, int *SndForm, char **Fname,
	    char **OutFile)
{
    int j = 1, pos, k;
    int getformat;
    char c;

#ifdef USEPLOTSOUND
    *PlotSnd = PLOTSOUND;
#endif
    *Verbose = VERBOSE;
    *Putkey = PUTKEY;
    *Backward = BACKWARD;
    *Divide = DIVIDE;
    *MinValue = -1;
    *SndValue = -1;
    *HeadSnd = HEADSND;
    *NullSnd = NULLSND;
    *TmpDir = TMPDIR;
    *UseMilliSec = USEMILLISEC;
    if ((*SndForm = setdefaultSndForm(argv[0])) == E_UNKNOWN) {
	getformat = 1;
	*SndForm = SNDFORMAT;
    } else getformat = 0;
    *SndCmd = auplay[*SndForm];
    *OutFile = OUTFNAME;

    if (argc < 2) return -30;
    c = *argv[j];
    if (c == '-') {
	c = *(++ argv[j]);
	while (1) {
	    if (c == 'h') { return 1; } /* Help */
	    if (c == 'v' || c == 'k' || c == 'e' || c == 'b'
#ifdef USEPLOTSOUND
	       || c == 'g'
#endif
		) { /* 1 ʸoption ʤ */
		switch (c) {
		case 'v': *Verbose = 1; break;
		case 'k': *Putkey = 1; break;
		case 'e': *Backward = 0; break;
		case 'b': *UseMilliSec = 0; break;
#ifdef USEPLOTSOUND
		case 'g': *PlotSnd = 1; break;
#endif
		}
		if ((c = *(++ argv[j])) != '\0') continue;
	    } else if (c == 'd' || c == 'm' || c == 'l' || c == 'o') {
		/* 1 ʸoption  */
		if (*(++ argv[j]) == '\0' && ++ j >= argc) return -1;
		switch (c) {
		case 'd' : 
		    if ((*Divide = atoi(argv[j])) < 2 
			|| *Divide > MaxDivide)
		      return -10;
		    break;
		case 'm':
		    if ((*MinValue = atof(argv[j])) <= 0.0) return -11;
		    break;
		case 'l':
		    if ((*SndValue = atoi(argv[j])) <= 0) return -20;
		    break;
		case 'o': *OutFile = argv[j]; break;
		}
	    }
	    else if (c == 'a' || c == 'w' || c == 'n') {
		/* 2 ʸʾ塢option ʤ */
		if (c == 'n') {
		    if (strcmp(argv[j], "nv") == 0) *Verbose = 0;
		    else if (strcmp(argv[j], "nk") == 0) *Putkey = 0;
		    else if (strcmp(argv[j], "ne") == 0) *Backward = 1;
		    else if (strcmp(argv[j], "nb") == 0)
		      *UseMilliSec = 1;
#ifdef USEPLOTSOUND
		    else if (strcmp(argv[j], "ng") == 0) *PlotSnd = 0;
#endif
		    else return -2;
		} else if (getformat) {
		    if (strcmp(argv[j], "au") == 0) *SndForm = E_AU;
		    else if (strcmp(argv[j], "wav") == 0)
		      *SndForm = E_WAV;
		    else if (strcmp(argv[j], "aiff") == 0)
		      *SndForm = E_AIFF;
		    else return -50;
		} else return -51;
	    } else if (c == 's' || c == 't') {
		/* 2 ʸʾ塢option  */
		if (strncmp(argv[j], "sh", 2) == 0) { 
		    pos = 2; k = E_SH; 
		} else if (strncmp(argv[j], "sn", 2) == 0) { 
		    pos = 2; k = E_SN; 
		} else if (strncmp(argv[j], "sp", 2) == 0) { 
		    pos = 2; k = E_SP; 
		} else if (strncmp(argv[j], "tmp", 3) == 0) {
		    pos = 3; k = E_TMP; 
		} else return -3;
		argv[j] += pos;
		if (*argv[j] == '\0' && ++ j >= argc) return -4;
		switch (k) {
		case E_SH: *HeadSnd = argv[j]; break;
		case E_SN: *NullSnd = argv[j]; break;
		case E_SP: *SndCmd = argv[j]; break;
		case E_TMP: *TmpDir = argv[j]; break;
		}
	    }
	    else return -70;
	    if (++ j >= argc) return -5;
	    if (*argv[j] != '-') break;
	    else c = *(++ argv[j]);
	}
    }
    if (j != argc - 1) return -6;
    *Fname = argv[j];
    return 0;
}

int setdefaultSndForm(char *commname)
{
    int len1, len2, j;

    len1 = strlen(commname);
    for (j=0; j<=2; j++) {
	if ((len2 = strlen(CommandNames[j])) <= len1) {
	    if (len2 == len1 && strcmp(CommandNames[j], commname) == 0)
	      return j;
	    if (len2 < len1 
		&& strcmp(CommandNames[j], commname + len1 - len2) == 0
		&& *(commname + len1 - len2 - 1) == '/') return j;
	}
    }
    return E_UNKNOWN;
}

#ifdef USEPLOTSOUND
/* ########## graph ʬ ########## */
/* pos[0] ~ pos[N-2], pos[N-1] ޤǽ
 * pos[N-1] 㳰
 * mode : 1 ==> MilliSec, 0 ==> block
 * mode2: 0 ==> 1 ==> plot, 2 ==> λ
 */
int plotcontrol(int mode, int mode2, char *fname, int N, int pos[],
		int size)
{
    int j, tmpsize = -1;
    double rest;
    char buff[MaxBuff];
    static int PlotDone = 0;
    static FILE *gp = NULL;
    int LINETYPE[] = {2, 3, 4, 5, 6, 7, 8, 9, 1}; 
    char *LABEL[] = {"1", "2", "3", "4", "5", "6", "7", "8", "9"};
    int LineTypeLast = -1;
    double REST = 0.03;

    if (PlotDone < 0) return PlotDone;
    if (mode2 == 0) { /*  */
	sprintf(buff, "%s %s -t .dat %s", SOX, fname, TMPDATFILE);
	system(buff);
	if ((tmpsize = getdatasize(TMPDATFILE)) <= 0
	   || (gp = popen(GNUPLOT, "w")) == NULL) {
	    if (tmpsize >= 0) remove(TMPDATFILE);
	    PlotDone = -1;
	    return PlotDone;
	}
	fprintf(gp, "set yrange [-1:1]\n");
	fprintf(gp, "set nokey\n");
	rest = (size - 0)*REST;
	if (mode) {
	    fprintf(gp, "set xrange [%f : %f]\n", blck2ms(-rest),
		    blck2ms(size + rest));
	    fprintf(gp, "plot '%s' using ($1*1000.0):2 w l 1\n",
		    TMPDATFILE);
	} else {
	    fprintf(gp, "set xrange [%f:%f]\n", - rest, size + rest);
	    fprintf(gp, "plot '%s' using ($0-1):2 w l 1\n", 
		    TMPDATFILE);
	}
	fflush(gp);
	PlotDone++;
	return PlotDone;
    } else if (mode2 > 0) { /* graph plot */
	if (PlotDone > 1) {
	    fprintf(gp, "set noarrow\n");
	    rest = (pos[N - 1] - pos[0])*REST;
	    if (mode)
	      fprintf(gp, "set xrange [%f:%f]\n", 
		      blck2ms(pos[0] - rest),
		      blck2ms(pos[N - 1] + rest));
	    else
	      fprintf(gp, "set xrange [%f:%f]\n", pos[0] - rest,
		      pos[N - 1] + rest);
	}
	for (j=1; j<N; j++) {
	    if (mode)
	      fprintf(gp, "set arrow from %f,-1 to %f,0.9 nohead lt %d\n",
		      blck2ms(pos[j - 1]), blck2ms(pos[j - 1]),
		      LINETYPE[j - 1]);
	    else
	      fprintf(gp, "set arrow from %d,-1 to %d,0.9 nohead lt %d\n",
		    pos[j - 1], pos[j - 1], LINETYPE[j - 1]);
	    if (*LABEL[j - 1] != '\0') {
		if (PlotDone > 1) fprintf(gp, "set nolabel %d\n", j);
		if (mode)
		  fprintf(gp, "set label %d '%s' at %f,0.93 center\n",
			  j, LABEL[j - 1], blck2ms(pos[j - 1]));
		else
		  fprintf(gp, "set label %d '%s' at %d,0.93 center\n",
			  j, LABEL[j - 1], pos[j - 1]);
	    }
	}
	if (mode)
	  fprintf(gp, "set arrow from %f,-1 to %f,1 nohead lt %d\n",
		  blck2ms(pos[N - 1]), blck2ms(pos[N - 1]),
		  LineTypeLast);
	else
	  fprintf(gp, "set arrow from %d,-1 to %d,1 nohead lt %d\n",
		  pos[N - 1], pos[N - 1], LineTypeLast);
	fprintf(gp, "replot\n");
	fflush(gp);
	PlotDone++;
	return PlotDone;
    } else { /* mode2 < 0 :  ǽ */
	if (PlotDone <= 0) return PlotDone;
	pclose(gp);
	remove(TMPDATFILE);
	PlotDone = 0;
	return PlotDone;
    }
}
#endif /* USEPLOTSOUND */

/* ##########  level ʲ ########## */
/* file size η׻ */
int getdatasize(char *fname)
{
    FILE *infile;
    int len;

    if ((infile = fopen(fname, "r")) == NULL) return -1;
    fseek(infile, 0L, SEEK_END);
    len = ftell(infile);
    fclose(infile);
    return len;
}


/* samplingrate μ */
int getsamplingrate(char *fname)
{
    char buff[MaxBuff];
    FILE *infile;
    int rate;

    sprintf(buff, "%s %s | awk '(NR == 2) {print $2}'", AUFINFO,
	    fname);
    if ((infile = popen(buff, "r")) == NULL) return -1;
    fgets(buff, MaxBuff, infile);
    rate = atoi(buff);
    pclose(infile);
    return rate;
}

/* sound data Ĺμ */
double getsoundlen(char *fname)
{
    char buff[MaxBuff];
    FILE *infile;
    double len;

    sprintf(buff, "%s %s | awk '(NR == 2) {print $6*1000.0/$2}'", 
	    AUFINFO, fname);
    if ((infile = popen(buff, "r")) == NULL) return -1;
    fgets(buff, MaxBuff, infile);
    len = atof(buff);
    pclose(infile);
    return len;
}

/* sound data size μ */
int getsoundblocks(char *fname)
{
    char buff[MaxBuff];
    FILE *infile;
    int len;

    sprintf(buff, "%s %s | awk '(NR == 2) {print $6}'", AUFINFO,
	    fname);
    if ((infile = popen(buff, "r")) == NULL) return -1;
    fgets(buff, MaxBuff, infile);
    len = atoi(buff);
    pclose(infile);
    return len;
}

/* ########## Usage ########## */
void usage(char *name)
{
    int j;

    fprintf(stderr, "Usage: %s ([options]) [datafile]\n", name);
    fprintf(stderr, " [datafile]: ԽϿե\n");
    fprintf(stderr, " [options]:\n");
    fprintf(stderr, "    -h: Υå\n");
#ifdef USEPLOTSOUND
    fprintf(stderr, "    -g/-ng: graph plot 򤹤/ʤ");
    fprintf(stderr, " (default: %s)\n", (PLOTSOUND) ? ("") 
	    : ("ʤ"));
#endif
    fprintf(stderr, "    -v/-nv: verbose Ϥ򤹤/ʤ");
    fprintf(stderr, " (default: %s)\n", (VERBOSE) ? ("")
	    : ("ʤ"));
    fprintf(stderr, "    -k/-nv: key assign ɽ򤹤/ʤ");
    fprintf(stderr, " (default: %s)\n", (PUTKEY) ? ("") 
	    : ("ʤ"));
    fprintf(stderr, "    -e/-ne: ̵ʹ/ʹ");
    fprintf(stderr, " (default: %s)\n", (BACKWARD) ? ("") 
	    : ("̵"));
    fprintf(stderr, "    -b/-nb: ñ̤ϥ֥å/ߥ");
    fprintf(stderr, " (default: %s)\n", (USEMILLISEC) ? ("ߥ")
	    : ("֥å"));
    
    if ((j = setdefaultSndForm(name)) == E_UNKNOWN) {
	fprintf(stderr, "    -au/-wav/-aiff:  format ");
	switch (SNDFORMAT) {
	case E_AU: fprintf(stderr, " (default: au)\n"); break;
	case E_WAV: fprintf(stderr, " (default: wav)\n"); break;
	case E_AIFF: fprintf(stderr, " (default: aiff)\n"); break;
	default: fprintf(stderr, " (default: ̤)\n"); break;
	}
	j = SNDFORMAT;
    }
    fprintf(stderr, "    -d [n]    : ʬ (2 <= n<=9, default: %d)\n",
	    DIVIDE);
    fprintf(stderr, "    -o [file] : ϥե (default: ɸ)\n");
    fprintf(stderr, "    -m [n]    : ǾĹ (ߥ/֥å:");
    fprintf(stderr,    " default: %f/%d)\n", MINLEN, MINSIZE);
    fprintf(stderr, "    -l [n]    : ץ벻κĹ (ߥ/֥å:");
    fprintf(stderr, " default: %d/%d)\n", SNDLEN, SNDBLOCK);
    fprintf(stderr, "    -tmp [dir]: ȥǥ쥯ȥ (default: %s)\n",
	    TMPDIR);
    fprintf(stderr, "    -sh [file]: ƲƬˤĤ벻ե̾");
    fprintf(stderr,    " (default: %s)\n", HEADSND);
    fprintf(stderr, "    -sn [file]: Ʋ̵ʬˤĤ벻ե̾");
    fprintf(stderr,    " (default: %s)\n", NULLSND);
    fprintf(stderr, "    -sp [comm]: եκޥ");
    fprintf(stderr, " (default: %s)\n", auplay[j]);
    fprintf(stderr, "1)  -m, - l ñ̤ -b/ - nb ˤäѹ롣\n");
    fprintf(stderr, "2)  -o  -o- ==> ɸ, -o+ ==> ̵, ");
    fprintf(stderr,    " λǤ롣\n");
}

#endif /* __AUSLCUT_C__ */
