/* ե벻ץ
 * shige
 * 10/29 2001
 * sndlevel.c
 *
 * $Id: sndlevel.c,v 1.11 2017-03-25 17:30:43+09 shige Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "common.h"

#define Leveldelimit ':'
#define Maxchannel 10

int main(int argc, char *argv[]);
int chbalancedlevel(int ch, double level[], double maxl[], 
                    double level_b);
double SNDgetmaxlevels(FILE *infile, SNDheader *snd, int format,
                       double maxlevel[], double *Level_B);
int getarg(int argc, char **argv, double level[], FILE **infile,
           FILE **outfile, int *Balance, int *Verbose);
int setlevel(char *str, double level[]);
void usage(int ret);

int main(int argc, char *argv[])
{
    SNDheader snd;
    FILE *infile, *outfile, *tmp = NULL;
    int ret, format, ch, bs, k, balance, verbose;
    double x, level[Maxchannel], maxl[Maxchannel], peek, level_b;
    long int j, blocks, dlen;

    ret = getarg(argc, argv, level, &infile, &outfile, &balance, 
		 &verbose);
    if (ret <= 0) { usage(ret); return -ret; }
#ifdef DEBUG2
    fprintf(stderr, "DEBUG: (balance, verbose) = (%d, %d)\n", 
	    balance, verbose);
    for (k=0; k<Maxchannel; k++)
      fprintf(stderr, "level[%d] = %f\n", k, level[k]);
#endif    
    if ((format = getSNDheader(infile, &snd)) < 0 
	|| format == E_UNKNOWN) {
	fprintf(stderr, "Error: getSNDheader(). (%d)\n", format);
	return 1;
    }

    dlen = SNDdatalen(&snd, format);
    if (infile == stdin) {
	if ((tmp = tmpfile()) == NULL) return 1;
	for (j=1; j<=dlen; j++) fputc(fgetc(infile), tmp);
	fclose(infile);
	if (fseek(tmp, 0L, SEEK_SET) < 0) return 1;
	infile = tmp;
    }
    if ((peek = SNDgetmaxlevels(infile, &snd, format, maxl, &level_b))
	<= 0.0) return 2;
    if (tmp == NULL) SNDskipheader(infile, format);
    else fseek(infile, 0L, SEEK_SET);

    ch = SNDchannels(&snd, format);

    if (verbose) {
#ifdef DEBUG1
	fprintf(stderr, "return = %d\n", ret);
#endif
	for (k=0; k<ch; k++)
	  fprintf(stderr, "(level[%d], maxl[%d]) = (%f, %f)\n", k, k,
		  level[k], maxl[k]);
#ifdef DEBUG1
	fprintf(stderr, " == ===\n");
	for (; k<Maxchannel; k++)
	  fprintf(stderr, "(level[%d], maxl[%d]) = (%f, %f)\n", k, k,
		  level[k], maxl[k]);
#endif
	fprintf(stderr, "(peek, level_b) = (%f, %f)\n", peek, level_b);
    }

    if (ret - 1 > ch || ch > Maxchannel) {
	fprintf(stderr, "Channel number error. (%d, %d, %d)\n",
		ret - 1, ch, Maxchannel);
	return 2;
    }

    putSNDheader(outfile, &snd, format);
    blocks = SNDblocks(&snd, format);
    bs = SNDblocksize(&snd, format);
    if (balance && ch > 1) {
	ret = chbalancedlevel(ch, level, maxl, level_b);
	if (ret < 0) return -ret;
    }
    for (j=1; j<=blocks; j++)
      for (k=0; k<ch; k++)
	if (level[k] < 0.0)
	  SNDpass1chrawdata(infile, outfile, &snd, format);
	else if (level[k] == 0.0 || maxl[k] <= 0.0)
	  SNDputnull1ch(outfile, &snd, format);
	else {
	    x = SNDget1chleveldata(infile, &snd, format);
	    x *= level[k]/maxl[k];
	    SNDput1chleveldata(outfile, x, &snd, format);
	}
    if (format != E_AU && bs%2 && blocks%2) fputc(0, outfile);
    return 0;
}

int chbalancedlevel(int ch, double level[], double maxl[], 
		    double level_b)
{
    int k;
    double x;

    if (ch <= 1) return 0;
    if (level_b <= 0.0) return -5;
    x = level[0];
    /* ٤Ƥλ٥뤬 */
    for (k=1; k<ch; k++) 
      if (x != level[k]) {
	  fprintf(stderr, 
		  "In balanced mode all level must be equal.\n");
	  return -6;
      }
    if (x <= 0.0) return 0;
    /* level_b  maxlevel Ȥƻ٥ˤ
     * ͥ 1 ۤʤ
     */
    for (k=0; k<ch; k++)
      if (maxl[k]*x/level_b > 1.0) {
	  fprintf(stderr, "%d channel level may over 1.0", k + 1);
	  fprintf(stderr, " (max, plevel) = (%f, %f)\n",
		  maxl[k], level_b);
	  return -7;
      }
    for (k=0; k<ch; k++) maxl[k] = level_b;
    return 0;
}


/* header  skip ϳ
 *  channel  level  channel ٥׻ channel ٥֤
 * common ˤƤۤΤä ?
 */
double SNDgetmaxlevels(FILE *infile, SNDheader *snd, int format,
		       double maxlevel[], double *Level_B)
{
    long int blocks, j;
    double x, max, xx;
    int ch, k;

    if (format != E_AU && format != E_WAV && format != E_AIFF)
      return -1.0;
    blocks = SNDblocks(snd, format);
    ch = SNDchannels(snd, format);

    if (blocks*ch < 1) return -1.0;
    
    x = SNDget1chleveldata(infile, snd, format);
    maxlevel[0] = Abs(x);
    if (ch > 1) {
	max = maxlevel[0];
	xx = x*x;
	for (k=1; k<ch; k++) {
	    x = SNDget1chleveldata(infile, snd, format);
	    maxlevel[k] = Abs(x);
	    if (maxlevel[k] > max) max = maxlevel[k];
	    xx += x*x;
	}
	*Level_B = xx;
    }
    for (j=2; j<=blocks; j++) {
	x = SNDget1chleveldata(infile, snd, format);
	if (Abs(x) > maxlevel[0]) maxlevel[0] = Abs(x);
	if (ch > 1) {
	    if (maxlevel[0] > max) max = maxlevel[0];
	    xx = x*x;
	    for (k=1; k<ch; k++) {
		x = SNDget1chleveldata(infile, snd, format);
		if (Abs(x) > maxlevel[k]) maxlevel[k] = Abs(x);
		if (maxlevel[k] > max) max = maxlevel[k];
		xx += x*x;
	    }
	    if (xx > *Level_B) *Level_B = xx;
	}
    }
    if (ch == 1) max = *Level_B = maxlevel[0];
    else *Level_B = sqrt(*Level_B/ch);
    return max;
}

int getarg(int argc, char **argv, double level[], FILE **infile,
	   FILE **outfile, int *Balance, int *Verbose)
{
    int j, maxchannel = 0;

    *outfile = stdout;
    *infile = stdin;
    *Balance = 0; *Verbose = 0;

    for (j=0; j<Maxchannel; j++) level[j] = -1.0;
    j = -1;
    while (-- argc > 0) {
	if (**(++ argv) == '-') {
	    switch (*(++ *argv)) {
	    case 'l':
		if (*(++ *argv) == '\0') { 
		    if (-- argc <= 0) return -1; 
		    else argv ++; 
		}
		if ((j = setlevel(*argv, level)) < 0) return -2;
		if (j > maxchannel) maxchannel = j;
		break;
	    case 'h':
	    case 'H':
	    case '?': return 0;
	    case 'o':
		if (*outfile != stdout) return -3;
		if (*(++ *argv) == '\0') {
		    if (-- argc <= 0) return -4;
		    else argv++; 
		}
		if (!strcmp(*argv, "-")) *outfile = stdout;
		else if ((*outfile = fopen(*argv, "w")) == NULL) {
		    fprintf(stderr, "File [%s] open error.\n", *argv);
		    return -5;
		}
		break;
	    case 'b': 
		*Balance = 1; break;
	    case 'v': 
		*Verbose = 1; break;
	    case '\0': break;
	    default: return -6;
	    }
	    if (**argv == '\0') { --*argv; break; }
	} else break;
    }
    if (argc > 1 || j < 0) return -7;
    else if (argc < 1) return maxchannel + 1;
    else if (!strcmp(*argv, "-") 
	     || (*infile = fopen(*argv, "r")) != NULL) 
      return maxchannel + 1;
    else {
	fprintf(stderr, "File [%s] open error.\n", *argv);
	return -9;
    }
}

/* 0 ~ 1  check 򤷤Ƥʤ
 * strtok ȤäǤ ?
 * ch = 0 ǧ뤫 
 * ch < 0 ǧ뤫 (ʳ) 
 */
int setlevel(char *str, double level[])
{
    int len = strlen(str), j, k, ch;
    char *s, *s1;
    double x;

    if (len <= 0) return -1;
    s = str; k = 0; ch = 0;
    for (j=1; j<=len; j++) {
	if (*s == Leveldelimit) {
	    if (++k >= 2) return -2;
	    s1 = s;
	}
	s++;
    }
    /* k == 0 ʤ ch = 0 Ʊ */
    if (k == 0) { ch = 0; x = atof(str); }
    else {
	/* k == 1 ʤ s ΰ֤ '\0' ˤͤ */
	if (*(s1 + 1) == '\0') return -3;
	if (*str == '\0') ch = 0;
	else if ((ch = atoi(str)) > Maxchannel || -ch > Maxchannel)
	  return -4;
	x = atof(s1 + 1);
    }
    if (x > 1.0) return -5; /* Υ٥ϲ⤷ʤȤǧ */
    if (ch == 0) {
	for (j=1; j<=Maxchannel; j++) level[j - 1] = x;
	return 0;
    } else if (ch > 0) { 
	level[ch - 1] = x; return ch; 
    } else {
	for (j=1; j<=Maxchannel; j++) if (j != -ch) level[j - 1] = x;
	return -ch;
    }
}

void usage(int ret)
{
    char *mess[] = {
	"Usage: sndlevel [option] ([infile])",
	" [option]:",
	"   -b: Х󥹥⡼ (default: ͥΥ٥)",
	"   -v: Ĺ (default: ĹϤʤ)",
	"   -o [outfile]: ϥեλ (default: ɸ)",
	"   -l[string]  : ̥٥λ (ʣ)",
	"   [infile]    : ϥե (ά: ɸ)",
	" ",
	" [1] ϥե '-' ꤷɸϤˤʤ롣",
	" [2] եκ粻̥٥ (sndstat -vl ɽ) ",
	"     ꤷ (0.0 ~ 1.0) ˤʤ褦˲̤ܤ롣",
	"     ꤷʤä channel ϲ⤷ʤ",
	" [3] ٥ϰʲ̤:",
	"       -l[level]     : ٤Ƥ channel κͤ [level] ",
	"       -l[n]:[level] : [n] channel κͤΤߤ [level] ",
	"     [n] ϥֹͥ桢[٥]  0.0 ~ 1.0 ϰϤμ¿",
	"     [n] < 0 ΤȤϡ|[n]| ͥʳλˤʤ롣",
	"     [level] < 0.0 ΤȤϤΥͥϲ⤷ʤ",
	" [4] -l ϸλ꤬λ񤭤롣ä㤨",
	"        -l0.9 -l1:0.5 -l1:0.7",
	"     ϡ1 channel  0.7¾ channel  0.9 ˤ롣",
	" [5] -b ϥХ󥹥⡼ɤǡ֤ͥΨݤä",
	"     ޤͥβ̤ܤ (default Ǥϥ",
	"     ͥΨϰۤʤ) Τǡͥΰۤʤ٥",
	"     Բġ",
	"     ޤξ 1 ֥åΥ٥򡢳ƥͥ",
	"     ͤ 2 ʿʿȤκ̤ͤκ",
	"     ȤƷ׻Τǡ1.0 ̤Υ٥Ǥ⤢",
	"     ͥ뤬 1.0 ۤ٥ˤʤꤦ뤬ξϥ",
	"     Ȥʤ (꾮ʥ٥ꤹɬפ)",
	NULL
    };
    int j;

    for (j=0; mess[j]!=NULL; j++) fprintf(stderr, "%s\n", mess[j]);
}
