/* 
 * resampling
 * shige
 * 01/06 2003
 * 
 * $Id: sndsmpl.c,v 1.10 2017-03-01 12:24:24+09 shige Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include "common.h"

#define Minsamples 50.0
#define Maxsamples 1000000.0

int main(int argc, char *argv[]);
void putmabiki(FILE *infile, FILE *outfile, SNDheader *snd, int format,
	       int channel, long int nblocks, int NN);
int getarg(int argc, char **argv, int *flag, double *N, FILE **infile,
	   FILE **outfile);
int isint(double x);
void usage(int ret);

int main(int argc, char *argv[])
{
    FILE *infile, *outfile;
    SNDheader snd, nsnd;
    int format, bs, ch, j;
    long int blocks, nblocks;
    double smpl, nsmpl, N, times;
    int ret, flag;
    double svol[MAXChannel], evol[MAXChannel];

    if ((ret = getarg(argc, argv, &flag, &N, &infile, &outfile)) <= 0) {
	usage(ret); return -ret;
    }
    if ((format = getSNDheader(infile, &snd)) < 0 
	|| format == E_UNKNOWN) {
        fprintf(stderr, "get header error.\n");
        return 3;
    }
    if ((smpl = SNDsmplrate(&snd, format)) <= 0.0) {
	fprintf(stderr, "sampling rate of data is not correct (%f).\n",
		smpl);
	return 5;
    }
    switch (flag) {
    case 0: /* -t: Ψ */
	nsmpl = smpl*N; break;
    case 1: /* -n: ¨ */
	nsmpl = N; break;
    case 2: /* -d: ְ */
	nsmpl = smpl/N; break;
    default: 
	fprintf(stderr, "Unknown mode. (%d)\n", flag); return 6;
    }
    if (nsmpl > Maxsamples || nsmpl < Minsamples) {
	fprintf(stderr, "value %f is too large sampling rate.\n", N);
	return 4;
    }
    bs = SNDblocksize(&snd, format);
    blocks = SNDblocks(&snd, format);
    ch = SNDchannels(&snd, format);
    cpSNDheader(&nsnd, &snd, format);
    if (flag != 2) {
	times = nsmpl/smpl;
	nblocks = (long int) (blocks*times + 0.5);
	if (setSNDheader(&nsnd, format, 0, 0, nsmpl, 0, nblocks,
			CH_SMPLRATE | CH_BLOCKS) < 0) {
	    fprintf(stderr, "setSNDheader error.\n");
	    return 5;
	}
	putSNDheader(outfile, &nsnd, format);
	for (j=0; j<ch; j++) svol[j] = evol[j] = 1.0;
	SNDputtrans(infile, outfile, &snd, format, nblocks, blocks, 
		    1.0/times, 0.0, 0.0, svol, evol);
    } else { /* ְ */
	long int NN = (long int)(N + 0.5); 
	
	nblocks = blocks/NN; if (blocks%NN) nblocks ++;
	if (setSNDheader(&nsnd, format, 0, 0, nsmpl, 0, nblocks,
			CH_SMPLRATE | CH_BLOCKS) < 0) {
	    fprintf(stderr, "setSNDheader error.\n");
	    return 5;
	}
	putSNDheader(outfile, &nsnd, format);
	putmabiki(infile, outfile, &snd, format, ch, nblocks, NN);
    }
    if (format != E_AU && bs%2 && nblocks%2) fputc(0, outfile);
    return 0;
}

/* ϰ̣Τְ򤹤ȤʿѤȤץ󥰤ΰ
 * 򤺤餹ȤǤ褦ˤȤ
 */
void putmabiki(FILE *infile, FILE *outfile, SNDheader *snd, int format,
	       int channel, long int nblocks, int NN)
{
    long int j;
    int k, m;
    double x;

    for (j=1; j<nblocks; j++) {
	for (m=1; m<=channel; m++) {
	    x = SNDget1chleveldata(infile, snd, format);
	    SNDput1chleveldata(outfile, x, snd, format);
	}
	for (k=1; k<=(NN-1)*channel; k++) 
	  SNDskip1chdata(infile, snd, format);
    }
    for (m=1; m<=channel; m++) {
	x = SNDget1chleveldata(infile, snd, format);
	SNDput1chleveldata(outfile, x, snd, format);
    }
}

int getarg(int argc, char **argv, int *flag, double *N, FILE **infile,
	   FILE **outfile)
{
    *outfile = stdout;
    *infile = stdin;
    *N= - 1.0;
    *flag = 1;

    while (-- argc > 0) {
	if (**(++ argv) == '-') {
	    switch (*(++ *argv)) {
	    case 'h':
	    case 'H':
	    case '?': return 0;
	    case 'o':
		if (*outfile != stdout) return -3;
		if (*(++ *argv) == '\0') { 
		    if (-- argc <= 0) return -4; else argv ++;
		}
		if (!strcmp(*argv, "-")) *outfile = stdout;
		else if ((*outfile = fopen(*argv, "w")) == NULL) {
		    fprintf(stderr, "File [%s] open error.\n", *argv);
		    return -5;
		}
		break;
	    case 't': *flag = 0; break;
	    case 'n': *flag = 1; break;
	    case 'd': *flag = 2; break;
	    default: return -6;
	    }
	}
	else break;
    }
    if (argc > 2 || argc < 1) {
#ifdef DEBUG1
	fprintf(stderr, "DEBUG1: %d\n", argc);
#endif
	return -7;
    }
    if ((*N = atof(*argv)) <= 0.0) return -8;
    if (*flag == 2 && ! isint(*N)) return -10;
    else if (argc == 1) return 1;
    else if (! strcmp(*(++ argv), "-")) *infile = stdin;
    else if ((*infile = fopen(*argv, "r")) == NULL) {
	fprintf(stderr, "File [%s] open error.\n", *argv);
	return -9;
    }
    return 1;
}

#define Mindecimal 0.00001

int isint(double x)
{
    double y = x - (long int) (x + 0.5);
#ifdef DEBUG1
    fprintf(stderr, "(x, y) = (%f, %f)\n", x, y);
#endif
    if (y > Mindecimal || y< -Mindecimal) return 0;
    else return 1;
}

void usage(int ret)
{
    char *mess[] = {
	"Usage: sndsmpl (-o [outfile]) (-t|-n|-d) [N] ([infile])",
	"   -o [outfile] : ϥեλ (default: ɸ)",
	"   -t : ߤ sampling rate  [N] ܤˤ",
	"   -d : ߤ sampling rate  1/[N] ˤ",
	"   -n : ߤ sampling rate  [N] ˤ (default)",
	"   [N] : sampling rate ",
	"   [infile] : ϥե (ά: ɸ)",
	" ",
	" [1] sampling rate ѹϡñ֤ˤäƹԤʤȷ",
	"     ѹʤ",
	" [2] ϥեϡꤷʤä硢ӥե̾Ȥ",
	"     '-' ꤷɸϤˤʤ롣",
	" [3] AU ե 8000Hz Τ褦ˡɤȤ sampling rate ʳ",
	"     ϥݡȤƤʤǥХ¿ȻפΤǡsampling",
	"     rate ѹΤǤʤǽ⤢롣",
	" [4] sampling rate 礭ꤹ file size 礭ʤΤ",
	"     뤳ȡߤ 1,000,000 /äۤȥ顼Ȥ롣",
	" [5] [N]  -t/-n ξμ¿-d ξȤ롣",
	NULL
    };
    int j;

#ifdef DEBUG1
    printf("USAGE: %d\n", ret);
#endif
    for (j=0; mess[j]!=NULL; j++)
      fprintf(stderr, "%s\n", mess[j]);
}
